/****** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is XpcomViewer code.
 *
 * The Initial Developer of the Original Code is
 * Ondrej Donek.
 * Portions created by Initial Developer are Copyright (C) 2005-2006
 * Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *  - Ondrej Donek, <ondrejd@gmail.com>, original author
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the LGPL or the GPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 ****** END LICENSE BLOCK *****/


//////////////////////////////////////////////////////////////////////////
// Constants

var XUL_XMLNS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";

var kRdfSrv = Components.classes ["@mozilla.org/rdf/rdf-service;1"].
		getService (Components.interfaces.nsIRDFService);

var kAtomSrv = Components.classes["@mozilla.org/atom-service;1"].
    getService (Components.interfaces.nsIAtomService);

var kPromptSrv = Components.classes["@mozilla.org/embedcomp/prompt-service;1"].
    getService (Components.interfaces.nsIPromptService);

var kPrefs = Components.classes ["@mozilla.org/preferences-service;1"].
    getService (Components.interfaces.nsIPrefService).
    getBranch ("extensions.xpcomviewer.");


var kDefaultWorkspaceView = "classes";
var kChromePathPrefix = "chrome://xpcomviewer/content/";
var kStatusbarIconRedURI = "chrome://xpcomviewer/skin/icons/16x16/progress-red.png";
var kStatusbarIconGreenURI = "chrome://xpcomviewer/skin/icons/16x16/progress-green.png";



//////////////////////////////////////////////////////////////////////////
// Global objects/variables

var gStrBundle = null;
var gBrandStrBundle = null;

/** Holds false until bookmarks/notes service is not initialized correctly */
var gAllowBookmarks = false;
var gAllowNotes = false;



//////////////////////////////////////////////////////////////////////////
// Common dialogs

/**  Show about dialog. */
function showAboutDialog () {
  var dlg = window.openDialog (kChromePathPrefix+"about.xul", "xpcomviewer:about",
      "chrome,extrachrome,dialog,modal,resizable=no,centerscreen");

  dlg.focus ();
}

/** Show preferences dialog. */
function showPreferences () {
	var dlg = window.openDialog (kChromePathPrefix+"preferences.xul",
	    "xpcomviewer:about", "chrome,extrachrome,toolbar,modal,resizable=no,centerscreen,all");

  dlg.focus ();
}

/** Opens toolbar customization dialog. */
function customizeToolbars () {
  var toolbox = document.getElementById ("xpcomviewer-toolbox");

  if (!toolbox) return;

	var dlg = window.openDialog ("chrome://global/content/customizeToolbar.xul",
	    "CustomizeToolbar", "chrome,all,dependent", toolbox);
  dlg.focus ();
}

/** Show XPCOMViewer help. */
function showHelp () {
  openHelp("xpcomviewer-help", "chrome://xpcomviewer/locale/help/help.rdf");
}

/** Show browser with XpcomViewer homepage */
function showHomePage () {
  alert ("Sorry, not implemented yet!");
}



//////////////////////////////////////////////////////////////////////////
// Misc. functions


/**
 * Reads and returns text from file
 *
 * @param  nsILocalFile
 * @return  string
 */
function readTextFromFile (aFile)
{
  var file = Components.classes["@mozilla.org/file/local;1"].
    createInstance (Components.interfaces.nsILocalFile);
  file.initWithFile (aFile);

  var text = new String ();
  var stream1 = null;
  stream1 = Components.classes["@mozilla.org/network/file-input-stream;1"].
    	createInstance (Components.interfaces.nsIFileInputStream);
  var stream2 = null;
  stream2 =  Components.classes["@mozilla.org/scriptableinputstream;1"].
    	createInstance (Components.interfaces.nsIScriptableInputStream);

  try {
    stream1.init (file, 1, 0, false);
    stream2.init (stream1);
    text += stream2.read (-1);

    if (stream2) stream2.close();
    if (stream1) stream1.close();
  } catch (e) {
    Components.utils.reportError (e + " " + file.path);
  }

  try {
	  var converter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].
	      createInstance (Components.interfaces.nsIScriptableUnicodeConverter);
	  converter.charset = "UTF-8";

  	return converter.ConvertToUnicode (text);
  } catch (e) {
    Components.utils.reportError (e);
    return text;
  }
} // end readTextFromFile (aFile)


/**
 * Save given text into file
 *
 * @param  nsIFile
 * @param  string
 */
function writeTextToFile (aFile, aText)
{
  var file = Components.classes["@mozilla.org/file/local;1"].
    createInstance (Components.interfaces.nsILocalFile);
  file.initWithFile (aFile);

  // Components.utils.reportError (file.path);

  var stream = null;
  stream = Components.classes["@mozilla.org/network/file-output-stream;1"].
      createInstance (Components.interfaces.nsIFileOutputStream);
  var os = Components.classes["@mozilla.org/intl/converter-output-stream;1"].
      createInstance (Components.interfaces.nsIConverterOutputStream);
  os.init (stream, "UTF-8", 0, 0x0000);

	try {
	  if (file.exists () == false)
	    aFile.create (0x00, 0777);

	  stream.init (file, 0x02 | 0x08 | 0x20, 0664, 0);
	  os.writeString (aText);

	  if (stream) stream.close ();
	  if (os) os.close ();
	} catch (e) {
	  Components.utils.reportError (e);
	}
} // end writeTextToFile (aFile, aText)


/**
 * Read RDF datasource to the file
 *
 * @param  nsIFile
 * @param  string
 */
function readDataSourceFromFile (aFile, aUri)
{
  var stream1 = Components.classes["@mozilla.org/network/file-input-stream;1"].
      createInstance (Components.interfaces.nsIFileInputStream);
  var stream2 = Components.classes["@mozilla.org/scriptableinputstream;1"].
      createInstance (Components.interfaces.nsIScriptableInputStream);
  var parser = Components.classes["@mozilla.org/rdf/xml-parser;1"].
      createInstance (Components.interfaces.nsIRDFXMLParser);

  var content = new String ("");
  var rootUri = Components.classes["@mozilla.org/network/io-service;1"].
      getService (Components.interfaces.nsIIOService).
      newURI (aUri, null, null);
  var ds = Components.classes["@mozilla.org/rdf/datasource;1?name=in-memory-datasource"].
      createInstance (Components.interfaces.nsIRDFDataSource);

  try {
    stream1.init (aFile, 1, 0, false);
    stream2.init (stream1);
    content += stream2.read (-1);

    var converter = Components.classes["@mozilla.org/intl/scriptableunicodeconverter"].
        createInstance (Components.interfaces.nsIScriptableUnicodeConverter);
    converter.charset = "UTF-8";

    parser.parseString (ds, rootUri, converter.ConvertToUnicode (content));
    stream2.close ();
    stream1.close ();
  } catch (e) {
    Components.utils.reportError (e);
    return false;
  }

  return ds;
} // readDataSourceFormFile (aFile, aUri)


/**
 * Write RDF datasource to the file
 *
 * @param  nsIFile
 * @param  nsIDataSource
 */
function writeDataSourceToFile (aFile, aDataSource)
{
  if (!(aFile.exists ())) aFile.create (0x00, 0664);

  var outputStream =
  {
    data : "",
    close : function () {},
    flush : function () {},
    write : function (buffer, count)  {
        this.data += buffer;
        return count;
    },
    writeFrom : function (stream, count) {},
    isNonBlocking : false
  }

  try {
	  var serializer = Components.classes["@mozilla.org/rdf/xml-serializer;1"].
	      createInstance(Components.interfaces.nsIRDFXMLSerializer);
	  var stream = Components.classes["@mozilla.org/network/file-output-stream;1"].
	      createInstance (Components.interfaces.nsIFileOutputStream);

	  stream.init (aFile, 0x02 | 0x08 | 0x20, 0664, 0);
	  serializer.init (aDataSource);
	  serializer.QueryInterface (Components.interfaces.nsIRDFXMLSource);
	  serializer.Serialize (stream);
	  stream.close ();
	} catch (e) {
	  Components.utils.reportError (e);
	}
} // writeDataSourceToFile (aFile, aDataSource)


/**
 * Create instance of nsIURI
 */
function createUri (aUrlString)
{
  try {
    var ioserv = Components.classes["@mozilla.org/network/io-service;1"].
        getService (Components.interfaces.nsIIOService);
    return ioserv.newURI (aUrlString, null, null);
  } catch (e) {
    Components.utils.reportError (e);
  }

  return null;
} // end createUri (aUrlString)


/**
 * Set up operating system
 */
function getOS ()
{
  if (gOS) return gOS;

  var platform = navigator.platform.toLowerCase();

  if (platform.indexOf("win") != -1) {
    gOS = "Win";
  } else if (platform.indexOf("mac") != -1) {
    gOS = "Mac";
  } else if (platform.indexOf ("unix") != -1 || platform.indexOf ("linux") != -1 ||
      platform.indexOf("sun") != -1) {
    gOS = "UNIX";
  } else {
    gOS = "";
  }

  return gOS;
} // end getOS ()


/**
 * Returns instance of nsIFile with Profile directory
 */
function getProfDir ()
{
	try {
		// @see mozilla/xpcom/io/nsDirectoryServiceDefs.h
		var file = Components.classes["@mozilla.org/file/directory_service;1"].
		    getService (Components.interfaces.nsIProperties).
		    get ("ProfD", Components.interfaces.nsIFile);

		file.append ("extensions");
		file.append ("xpcomviewer@ondrejd.iglu.cz");
  } catch (e) {
    Components.utils.reportError (e);
    file = false;
  }

	return file;
}



//////////////////////////////////////////////////////////////////////////
// XpcomViewer tree variables

/**
 * These variables are used by all tree views.
 * For easy creating new treeview us newTreeView object prototype
 * and for treeitems use fucntion newTreeItem which creates
 * new treeitem object for you.
 *
 * @todo Make prototype of TreeView and share it between views too
 * @todo Global sort methods
 */

/** Constant with names of single treeviews */
var kTreeViewsNames = { classes : "Components.classes",
                        ifaces  : "Components.interfaces",
                        results : "Components.results" };

/** Holds pointer to xpcomviewer tree */
var gXpcomTree = null;

/** Holds name of currently displayed treeview */
var gCurrentTreeView = "";

/** Array of items for current treeview */
var gXpcomTreeView = [];

/** Object that holds backups of all three views */
var gXpcomTreeViewBackup = { classes : [],
                             ifaces  : [],
                             results : [] };

/** Type of used filter */
var gXpcomTreeFilter = "none"; // none | containing | starts_with |
                               // with_note | with_bookmark

/** Holds last treeview (before apply filter) */
var gXpcomTreeFilterBackup = [];

/** Holds informations about if tree is sorted and how.  */
var gXpcomTreeSort = { sortedBy : "", // contains name of col
                       direction : "natural" }; // natural | ascending | descending


/**
 * Create new tree item
 */
function newTreeItem (aName, aValue, aType, aLevel, aIsContainer)
{
  var item = new Object ();

  item.mBookmarkCol = false;
	if (gAllowBookmarks) {
	  if (gBookmarksDataSource.hasItemBookmark (aName)) {
	    item.mBookmarkCol = true;
	  }
	}

  item.mNoteCol = false;
	if (gAllowNotes) {
	  if (gNotesDataSource.hasItemNote (aName)) {
	    item.mNoteCol = true;
	  }
	}

	item.mNameCol = aName;
	item.mValueCol = aValue;
	item.mType = aType;
	item.mLevel = aLevel;
	item.mIsContainer = aIsContainer;
	item.mIsContainerOpen = false;
	item.mIsContainerEmpty = true;
	item.mHiddenRows = [];

	return item;
} // End of newTreeItem ();



/**
 * Tree view prototype object
 */
function newTreeView () {}

newTreeView.prototype.treeBox = null;
newTreeView.prototype.selection = null;
newTreeView.prototype.__defineGetter__ ("rowCount", function () { return gXpcomTreeView.length; });

newTreeView.prototype.getCellText = function (aRow, aCol) {
  switch (aCol.id) {
    case "nameCol":
      return gXpcomTreeView[aRow].mNameCol;
      break;
    case "valueCol":
      return gXpcomTreeView[aRow].mValueCol;
      break;
  }

  return "";
};

newTreeView.prototype.getCellValue = function (aRow, aCol) {
  switch (aCol.id) {
    case "bookmarkCol":
      return gXpcomTreeView[aRow].mNoteCol;
      break;

    case "noteCol":
	    if (gAllowNotes) {
	      if (gNotesDataSource.hasItemNote (gXpcomTreeView [aRow].mNameCol)) {
	      	gXpcomTreeView[aRow].mNoteCol = true;
	      }
	    }
      return gXpcomTreeView[aRow].mNoteCol;
      break;
  }

  return null;
};

newTreeView.prototype.getRowProperties = function (aRow, aProp) { return; };

newTreeView.prototype.getCellProperties = function (aRow, aCol, aProp) {
  switch (aCol.id) {
    case "bookmarkCol":
      if (gXpcomTreeView [aRow].mBookmarkCol) {
        aProp.AppendElement (kAtomSrv.getAtom ("hasBookmark"));
      } else {
        aProp.AppendElement (kAtomSrv.getAtom ("noFlag"));
      }
      break;

    case "noteCol":
      if (gXpcomTreeView [aRow].mNoteCol) {
        aProp.AppendElement (kAtomSrv.getAtom ("hasNote"));
      } else {
        aProp.AppendElement (kAtomSrv.getAtom ("noFlag"));
      }
      break;

    case "nameCol":
      if (gXpcomTreeView [aRow].mType == "interface" &&
         (gXpcomTreeView [aRow].mNameCol == "nsISupports" ||
          gXpcomTreeView [aRow].mNameCol == "nsIClassInfo")) {
        aProp.AppendElement (kAtomSrv.getAtom ("constIface"));
      } else if (gXpcomTreeView [aRow].mType == "method" &&
          gXpcomTreeView [aRow].mNameCol == "QueryInterface") {
         aProp.AppendElement (kAtomSrv.getAtom ("constMethod"));
      } else {
        aProp.AppendElement (kAtomSrv.getAtom (gXpcomTreeView[aRow].mType));
      }
      break;
  }
};

newTreeView.prototype.getColumnProperties = function (aCol, aProp) {};

newTreeView.prototype.isContainer = function (aRow) {
  return gXpcomTreeView [aRow].mIsContainer;
};

newTreeView.prototype.isContainerOpen = function (aRow) {
  return (!(this.isContainer (aRow)))
      ? false
      : gXpcomTreeView[aRow].mIsContainerOpen;
};

newTreeView.prototype.isContainerEmpty = function (aRow) {
  return (!(this.isContainer (aRow)))
      ? false
      : gXpcomTreeView[aRow].mIsContainerEmpty;
};

newTreeView.prototype.canDrop = function (aRow, aOrientation) { return true; };
newTreeView.prototype.drop = function (aRow, aOrientation) {};
newTreeView.prototype.setTree = function (aOut) { this.treeBox = aOut; };

newTreeView.prototype.getParentIndex = function (aRow) {
  for (var i = aRow-1; i >= 0; i--) {
    if (this.getLevel (i) < this.getLevel (aRow)) return i;
  }

  return -1;
};

newTreeView.prototype.hasNextSibling = function (aRow, aAfterIndex) {
	var thisLevel = this.getLevel (aRow);

	for (var t = aRow + 1; t < gXpcomTreeView.length; t++) {
	  var nextLevel = this.getLevel (t);

	  if (nextLevel == thisLevel)
	    return true;
	  else if (nextLevel < thisLevel)
	    return false;
	}

	return false;
};

newTreeView.prototype.getLevel = function (aRow) {
  return gXpcomTreeView[aRow].mLevel;
};

newTreeView.prototype.getImageSrc = function (aRow, aCol) { return ""; };
newTreeView.prototype.toggleOpenState = function (aIndex) {};
newTreeView.prototype.isSorted = function () { return true; };

newTreeView.prototype.cycleHeader = function (aCol) {
	if (aCol.id == "valueCol") {
	  gStatusbar.updateLabel ("Tree doesn't support sorting of this column.");
	}

	switch (aCol) {
	  case "noteCol":
	  case "bookmarkCol":
      // ..
	    break;

	  case "nameCol":
      // ..
	    break;
	}
};

newTreeView.prototype.selectionChanged = function () {};

newTreeView.prototype.cycleCell = function (aRow, aCol) {
  switch (aCol.id) {
    case "noteCol":
      if (!(gXpcomTreeView[aRow].mNoteCol)) {
        var dlg = window.openDialog (kChromePathPrefix+"notedialog.xul",
            "xpcomviewer:notedialog", "chrome,extrachrome,modal,resizable=no,centerscreen,all");
        dlg.focus ();

        // gStatusbar.updateLabel ("Inserting note at [" +
        //     gXpcomTreeView[aRow].mNameCol + "]");
        // this.setCellValue (aRow, aCol, true);
      } else {
        gStatusbar.updateLabel ("Deleting note at [" +
            gXpcomTreeView[aRow].mNameCol + "]");
        this.setCellValue (aRow, aCol, false);
      }

      break;

    case "bookmarkCol":
      if (!(gXpcomTreeView[aRow].mBookmarkCol)) {
        var res = gBookmarksDataSource.add (gXpcomTreeView[aRow].mNameCol);
        if (!(res)) {
          alert ("Bookmark adding failed!\nPlease, try again.");
          return;
        }

        this.setCellValue (aRow, aCol, true);
      } else {
        var res = gBookmarksDataSource.remove (gXpcomTreeView[aRow].mNameCol);
        if (!(res)) {
          alert ("Bookmark removing failed!\nPlease, try again.");
          return;
        }

        this.setCellValue (aRow, aCol, false);
      }
      break;
  }
};

newTreeView.prototype.isEditable = function (aRow, aCol) {
  return (aCol.id == "noteCol" || aCol.id == "bookmarkCol") ? true : false;
};

newTreeView.prototype.setCellText = function (aRow, aCol, aValue) {};

newTreeView.prototype.setCellValue = function (aRow, aCol, aValue) {
  switch (aCol.id) {
    case "noteCol":
      Components.utils.reportError ("Set cell value [noteCol]"+aValue);
      gXpcomTreeView[aRow].mNoteCol = aValue;
      break;

    case "bookmarkCol":
      Components.utils.reportError ("Set cell value [bookmarkCol]"+aValue);
      gXpcomTreeView[aRow].mBookmarkCol = aValue;
      break;
  }
};
newTreeView.prototype.performAction = function (aAction) {};
newTreeView.prototype.performActionOnRow = function (aAction, aRow) {};
newTreeView.prototype.preformActionOnCell = function (aAction, aRow, aCol) {};
newTreeView.prototype.isSeparator = function (aIndex) { return false; };



//////////////////////////////////////////////////////////////////////////
// Statusbar

var gStatusbarLabel = null;
var gStatusbarIcon = null;


/**
 * Statusbar helper object
 *
 * @todo updateLabel: add (by user preference configurable) delay
 *       after that will be label automatically erased
 * @fix updateIcon: doesn't work properly
 */
var gStatusbar = {

	updateLabel : function (aText)
	{
		if (gStatusbarLabel) {
		  gStatusbarLabel.setAttribute ("label", aText);
		}
	},

	updateIcon : function (aSrc)
	{
		if (gStatusbarIcon) {
		  gStatusbarIcon.setAttribute ("src", aSrc);
		}
	}
} // End of gStatusbar



//////////////////////////////////////////////////////////////////////////
// Bookmarks/Notes helper functions


/**
 * Initialize bookmarks datasource
 */
function startBookmarkService ()
{
	var bookmarksFile = Components.classes["@mozilla.org/file/local;1"].
      createInstance (Components.interfaces.nsILocalFile);

  bookmarksFile.initWithFile (getProfDir ());
  bookmarksFile.append ("bookmarks.rdf");

	var res = gBookmarksDataSource.init (bookmarksFile, "xpcomviewer:bookmarks");

	if (!(res)) {
	  alert ("Bookmarks service initialization failed.");
	  disableBookmarkService ();
	  return;
	}

  gAllowBookmarks = true;
} // end startBookmarkService ()


/**
 * Stop (and save) bookmarks datasource
 */
function stopBookmarkService ()
{
  if (gAllowBookmarks) {
    gBookmarksDataSource.save ();
    disableBookmarkService ();
  }
} // end stopBookmarkService ()


/**
 * Disable bookmarks service
 */
function disableBookmarkService ()
{
	var popupItem = document.getElementById ("addBookmark-popupItem");
	var popupSep = document.getElementById ("popupBookmarkNoteSeparator");
	var toolbarBtn = document.getElementById ("bookmark-button");
	var toolbarSep = document.getElementById ("toolbarBookmarkNoteSeparator");
	var filterCmd = document.getElementById ("bookmarks-filter");
	var filterSep = document.getElementById ("popupBookmarksNotesFilter");
	var treeCol = document.getElementById ("bookmarkCol");

	popupItem.setAttribute ("hidden", true);
	filterCmd.setAttribute ("hidden", true);
	toolbarBtn.setAttribute ("hidden", true);
	treeCol.setAttribute ("hidden", true);

	if (document.getElementById ("addNote-popupItem").hasAttribute ("hidden"))
	  popupSep.setAttribute ("hidden", true);

	if (document.getElementById ("note-button").hasAttribute ("hidden"))
	  toolbarSep.setAttribute ("hidden", true);

	if (document.getElementById ("notes-filter").hasAttribute ("hidden"))
	  filterSep.setAttribute ("hidden", true);

	gAllowBookmarks = false;
	gStatusbar.updateLabel ("Bookmarks service disabled.");
} // end disableBookmarkService ()


/**
 * Initialize notes datasource
 */
function startNoteService ()
{
	var notesFile = Components.classes["@mozilla.org/file/local;1"].
      createInstance (Components.interfaces.nsILocalFile);

  notesFile.initWithFile (getProfDir ());
  notesFile.append ("notes.rdf");

	var res = gNotesDataSource.init (notesFile, "xpcomviewer:notes");

	if (!(res)) {
	  alert ("Notes service initialization failed.");
	  disableNoteService ();
	}

	gAllowNotes = true;
} // end startNoteService ()


/**
 * Stop (and save) notes datasource
 */
function stopNoteService ()
{
  if (gAllowNotes) {
    gBookmarksDataSource.save ();
    disableBookmarkService ();
  }
} // end stopNoteService ()


/**
 * Disable notes service
 */
function disableNoteService ()
{
	var popupItem = document.getElementById ("addNote-popupItem");
	var popupSep = document.getElementById ("popupBookmarkNoteSeparator");
	var toolbarBtn = document.getElementById ("note-button");
	var toolbarSep = document.getElementById ("toolbarBookmarkNoteSeparator");
	var filterCmd = document.getElementById ("notes-filter");
	var filterSep = document.getElementById ("popupBookmarksNotesFilter");
	var treeCol = document.getElementById ("noteCol");

	popupItem.setAttribute ("hidden", true);
	toolbarBtn.setAttribute ("hidden", true);
	filterCmd.setAttribute ("hidden", true);
	treeCol.setAttribute ("hidden", true);

	if (document.getElementById ("addBookmark-popupItem").hasAttribute ("hidden"))
	  popupSep.setAttribute ("hidden", true);

	if (document.getElementById ("bookmark-button").hasAttribute ("hidden"))
	  toolbarSep.setAttribute ("hidden", true);

	if (document.getElementById ("bookmarks-filter").hasAttribute ("hidden"))
	  filterSep.setAttribute ("hidden", true);

	gAllowNotes = false;
	gStatusbar.updateLabel ("Notes service disabled.");
} // end disableNoteService ()



//////////////////////////////////////////////////////////////////////////
// Variables and helper prototype object for RDFDataSource object


/**
 * Datasource service prototype
 *
 * @param string    Module name (e.g. 'pages')
 * @param string    Default type (XXX Remove this!!!)
 * @param nsIFile   Pointer to file with datasource
 * @param string    Base uri of RDF datasource
 */
function newDataSource ()
{
  this._ds = null;
  this._dsFile = Components.classes["@mozilla.org/file/local;1"].
      createInstance (Components.interfaces.nsILocalFile);
  this._dsFileName = "";
  this._namespace = "http://www.ondrejd.iglu.cz/xpcomviewer#";
  this._uriBase = "";
  this._resBase = null;
  this._rootCont = Components.classes["@mozilla.org/rdf/container;1"].
      createInstance (Components.interfaces.nsIRDFContainer);

  this.initialized = false;
}

// Some usefull RDF related methods
newDataSource.prototype._getArc = function (aArcName) {
  return kRdfSrv.GetResource (this._namespace + aArcName);
};

newDataSource.prototype._getLiteral = function (aValue) {
  return kRdfSrv.GetLiteral (aValue);
};

newDataSource.prototype._getTarget = function (aUri, aArcName) {
  var resource = kRdfSrv.GetResource (aUri);
  var target = this._ds.GetTarget (resource, this._getArc (aArcName), true);

  if (!(target instanceof Components.interfaces.nsIRDFLiteral)) return false;

  return (target.Value == "undefined") ? false : target.Value;
};

// Base methods

/**
 * Initialize datasource service
 *
 * @todo What about datasource observer?
 */
newDataSource.prototype.init = function (aFile, aUri)
{
  if (!(aFile instanceof Components.interfaces.nsIFile)) {
    alert ("newDataSource.init :: aFile is not instance of nsIFile");
    return false;
  }

  this._dsFileName = aFile.leafName;
  this._uriBase = aUri;
  this._resBase = kRdfSrv.GetResource (this._uriBase);

  try {
    this._dsFile.initWithFile (aFile);
    if (!(this._dsFile.exists ())) this.createNew (); // create new datasource

    this._ds = readDataSourceFromFile (this._dsFile, this._uriBase);
    this._rootCont.Init (this._ds, this._resBase);

    this.initialized = true;
  } catch (e) {
    Components.utils.reportError (e);

    this.initialized = false;
  }

  return this.initialized;
};

newDataSource.prototype.createNew = function () {
	this._ds = Components.classes["@mozilla.org/rdf/datasource;1?name=in-memory-datasource"].
	    createInstance (Components.interfaces.nsIRDFDataSource);

	// initialize root container
	this._rootCont = Components.classes["@mozilla.org/rdf/container-utils;1"].
	    createInstance (Components.interfaces.nsIRDFContainerUtils).
	    MakeSeq (this._ds, this._resBase);

	this.save ();
}

newDataSource.prototype.save = function () {
  writeDataSourceToFile (this._dsFile, this._ds);
};

newDataSource.prototype.add = function (aValues) {
  var newItem = kRdfSrv.GetResource (this._uriBase + ":" +
      Math.round (Math.random () * 64 * 32 * 8));

  for (var i=0; i<aValues.length; i++) {
    this._ds.Assert (newItem, this._getArc (aValues[i].label),
        kRdfSrv.GetLiteral (aFilename), true);
  }

  this._rootCont.AppendElement (newItem);
};

newDataSource.prototype.remove = function (aUri) {
  this._rootCont.RemoveElement (kRdfSrv.GetResource (aUri), false);
};

newDataSource.prototype.addProperty = function (aUri, aProperty) {
  var resource = kRdfSrv.GetResource (aUri);
  var lit = this._getLiteral (prompt ("Value for " + aProperty + ":"));

  this._ds.Assert (resource, kRdfSrv.GetResource (this._namespace + aProperty), lit, true);
};

newDataSource.prototype.changeProperty = function (aUri, aPropTitle, aPropValue) {
  var resource = kRdfSrv.GetResource (aUri);
  var target = this._ds.GetTarget (resource, this._getArc (aPropTitle), true);

  if (!(target instanceof Components.interfaces.nsIRDFLiteral)) return false;
  if (target.Value == "undefined") return false;

  this._ds.Change (resource, this._getArc (aPropTitle), target,
      this._getLiteral (aPropValue));

  return true;
};

newDataSource.prototype.removeProperty = function (aUri, aProperty) {
  var resource = kRdfSrv.GetResource (aUri);
  var target = this._ds.GetTarget (resource,
      kRdfSrv.GetResource (this._namespace + aProperty), true);

  this._ds.Unassert (resource,
      kRdfSrv.GetResource (this._namespace + aProperty), target);
};
